// Main button polling and event processing loop
// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#include <8052.h>
#include <stdio.h>

#include "buttons.h"
#include "../wincomm/shared.h"

#define BINARY_COMM

idata event clicks[NUMBUTTONS];
idata unsigned int mask = BUTTONMASK;
bit system_state=STATE_STOP, start_fail = 0, ser_timer_avail = 0;
bit send_stop_msg=0;
idata unsigned long ser_timer;
idata unsigned int fail_ports;


void main(void)
{
  idata unsigned char idx, led_timer=0;
  idata unsigned int lmask, min_evt;
  unsigned long tmin = 0;
  
  // Reset everything
  ClrTimer();
  
  hwinit();

  while (1)
  {
    if (system_state == STATE_RUN)
    {
      // Some buttons were already pushed when we started
      if (start_fail)
      {
        start_fail = 0;
         send_msg(MCR_E_STARTFAIL, fail_ports, 0);
      }
      // Read the mask
      lmask = mask; min_evt=0; tmin = 0;
      for (idx=0; idx<NUMBUTTONS; idx++)
      {
        if (lmask&1) // There is a new event
        {
          if ((tmin==0) || (tmin > clicks[idx].timer))
          {
            tmin=clicks[idx].timer;
            min_evt |= (unsigned int)1<<idx;
          }
          else if (tmin == clicks[idx].timer)
            min_evt |= (unsigned int)1<<idx;
        }
        lmask >>=1;
      }
      
      if (min_evt)
      {
	send_msg(MCR_E_EVENT, min_evt, tmin);

        // Disable interrupts to reset event mask
        EA = 0;
        mask &= ~min_evt;
        EA = 1;
      }
      if (ser_timer_avail)
      {
        ser_timer_avail = 0;
        send_msg(MCR_E_SERTIMER, 0, ser_timer);
      }
      
      // Do fast LED blinking
      idx = (unsigned char)((RdTimer()>>LED_C_FAST)&0xff);
      if (idx-led_timer)
      {
        P2_4 = ~P2_4;
        led_timer = idx;
      }
    }
    else
    {
      if (send_stop_msg)
      {
        send_stop_msg=0;
        send_msg(MCR_E_STOP, 0, RdTimer());
      }
      // Do slow LED blinking
      idx = (unsigned char)((RdTimer()>>LED_C_SLOW)&0xff);
      if (idx-led_timer)
      {
        P2_4 = ~P2_4;
        led_timer = idx;
      }
    }    
  }
}
