// Serial communication routines
// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#include <8052.h>
#include <stdio.h>

#include "buttons.h"

// Host interface definitions
#include "../wincomm/shared.h"

extern idata unsigned int mask;
extern bit system_state, start_fail, ser_timer_avail, send_stop_msg;
extern idata unsigned long ser_timer;
extern idata unsigned int fail_ports;
extern idata unsigned int old_ports;

void ser_handler (void) interrupt 4 using 2
{
  unsigned char ch;

  EA = 0;
    
  if (RI)
  {
    RI = 0;
    
    ch = SBUF;
    
    switch (ch)
    {
      // Start the processing
      case MCR_C_RESTART:
        system_state = STATE_RUN;
        ser_timer_avail = 0;
        start_fail = 1;
	mask = 0; old_ports=0;
        ClrTimer();
        
        // Check if any button is already pushed
        fail_ports = P0 | ((unsigned int)P2 << 8);
        fail_ports &= BUTTONMASK;
        break;
      case MCR_C_STOP:
        system_state = STATE_STOP;
        send_stop_msg = 1;
        break;
      case MCR_C_GETTIMER:
        ser_timer = RdTimer();
        ser_timer_avail = 1;
    }
  }
  EA = 1;
}


void ser_putc (char c)
{
  SBUF = c;
  while (!TI)
    ;
  TI = 0;
}

void ser_puts (char *str)
{
  unsigned char c;
  while (c = *str++)
  {
    if (c == '\n')
      ser_putc ('\r');
    ser_putc (c);
  }
}

void putchar (char c)
{
  ser_putc (c);
}

#define BINARY_COMM

void send_msg(unsigned int evtType, unsigned int mask, unsigned long timer)
{
  #ifdef BINARY_COMM
  mask |= evtType;
  ser_putc((unsigned char)((mask>>8)&0xff));
  ser_putc((unsigned char)(mask&0xff));
  ser_putc((unsigned char)((timer>>24)&0xff));
  ser_putc((unsigned char)((timer>>16)&0xff));
  ser_putc((unsigned char)((timer>>8)&0xff));
  ser_putc((unsigned char)(timer&0xff));
  #else
  switch (evtType)
  {
    case MCR_E_EVENT:
      ser_puts("Button event");
      break;
    case MCR_E_STARTFAIL:
      ser_puts("Start fail");
      break;
    case MCR_E_SERTIMER:
      ser_puts("Serial timer");
      break;
    case MCR_E_STOP:
      ser_puts("Stop event");
  }    
  printf_fast(": mask 0x%x, timer %lu\n\r", mask, timer);
  #endif
}
