// MINI/MAX-51C interface functions
// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#include "../stdafx.h"
#include <mmsystem.h>

#include "serial.h"
#include "mcr.h"
#include "shared.h"

static bool mcrGetSingle(HANDLE *hCom, mcrEvent *evt);

static mcrEvent evtBuf[MCR_C_MSGBUF];
static int buf_ptr=0;

void mcrAddMsg(HANDLE *hCom, int button)
{
	unsigned long timer;
	if(hCom)
		mcrGetTimer(hCom, &timer);
	else
		timer = timeGetTime();

	mcrEvent evt;
	evt.mask   = 1 << button;
	evt.timer = timer * 10;
	evt.evType = MCR_M_EVENT;

	memcpy(evtBuf+buf_ptr++, &evt, sizeof(mcrEvent));
}

bool mcrCheckMsg(HANDLE *hCom)
{
	if (buf_ptr)
		return true;

	if (SerialInBuf(hCom))
		return true;
	else
		return false;
}

bool mcrGetTimer(HANDLE *hCom, unsigned long *timer)
{
	__int64 t0, dt;
	mcrEvent evt;

	if(bKeyboard){
		*timer = timeGetTime();
		return true;
	}

	SerialPutc(hCom, MCR_C_GETTIMER);

	t0 = (__int64)timeGetTime();
	do
	{
		if (mcrGetSingle(hCom, &evt))
		{
			if (evt.evType == MCR_E_SERTIMER)
			{
				*timer = evt.timer;
				return true;
			}
			else
				// Add the event to the buffer
				memcpy(evtBuf+buf_ptr++, &evt, sizeof(mcrEvent));
		}
		dt = (__int64)timeGetTime()-t0;
		if (dt < 0)	dt += (__int64)UINT_MAX+1;
	} while (dt < MCR_T_GETTIMER);

	return false;
}

bool mcrRestart(HANDLE *hCom, unsigned short *mask)
{
	__int64 t0, dt;
	mcrEvent evt;

	if(bKeyboard)
		return true;

	SerialPutc(hCom, MCR_C_RESTART);

	t0 = (__int64)timeGetTime();
	do
	{
		if (mcrGetSingle(hCom, &evt))
		{
			if (evt.evType == MCR_E_STARTFAIL)
			{
				*mask = evt.mask;
				return true;
			}
			else
				// Add the event to the buffer
				memcpy(evtBuf+buf_ptr++, &evt, sizeof(mcrEvent));
		}
		dt = (__int64)timeGetTime()-t0;
		if (dt < 0)	dt += (__int64)UINT_MAX+1;
	} while (dt < MCR_T_RESTART);

	return false;
}

static bool mcrGetSingle(HANDLE *hCom, mcrEvent *evt)
{
	unsigned char buf[MCR_C_MSGSIZE], *p = buf;
	unsigned long cnt;

	cnt = SerialRead(hCom, (void *)buf, MCR_C_MSGSIZE);
	if (cnt != MCR_C_MSGSIZE)
		return false;
	
	p = buf;
	evt->mask   = (unsigned short)(*p++)<<8;
	evt->mask  |= *p++;
	evt->timer  = (unsigned long)(*p++)<<24;
	evt->timer |= (unsigned long)(*p++)<<16;
	evt->timer |= (unsigned long)(*p++)<<8;
	evt->timer |= *p;
	evt->evType = evt->mask & MCR_M_EVENT;
	evt->mask &= ~MCR_M_EVENT;
	return true;
}

bool mcrGetMsgs(HANDLE *hCom, mcrEvent **evt, int *count)
{
	bool rc = true;
	mcrEvent *p;

	while (SerialInBuf(hCom))
	{
		if (!mcrGetSingle(hCom, evtBuf+buf_ptr++))
		{
			buf_ptr--;
			rc = false;
			break;
		}
	}

	*count = buf_ptr;
	buf_ptr = 0;

	if ((p = (mcrEvent *)malloc(*count * sizeof(mcrEvent)))==NULL)
	{
		*count = 0; *evt = NULL;
		return false;
	}

	memcpy(p, evtBuf, *count * sizeof(mcrEvent));

	*evt = p;
	return rc;
}

bool mcrStop(HANDLE *hCom)
{
	__int64 t0, dt;
	mcrEvent evt;

	SerialPutc(hCom, MCR_C_STOP);

	t0 = (__int64)timeGetTime();
	do
	{
		if (mcrGetSingle(hCom, &evt))
			if (evt.evType == MCR_E_STOP)
			{
				buf_ptr = 0;
				return true;
			}
			// Discard other events
		
		dt = (__int64)timeGetTime()-t0;
		if (dt < 0)	dt += (__int64)UINT_MAX+1;
	} while (dt < MCR_T_STOP);

	return false;
}
