// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#ifndef _MCR_H_
#define _MCR_H_

// Buffer up to 100 messages
#define MCR_C_MSGBUF	100

// Timeouts in ms
#define MCR_T_GETTIMER	200
#define MCR_T_RESTART	200
#define MCR_T_STOP		200

// total number of buttons
#define MCR_BUTTONS		12 

// Event linked-list structure
typedef struct _mcrEvent
{
	// Event type, see shared.h
	unsigned short evType;
	// A mask of pressed buttons, bits 0-11
	unsigned short mask;
	// Event timer (a 0-to-1 transition detected), each
	// timer tick is 100 microseconds
	unsigned long timer;
} mcrEvent;

/*
   Returns true if a message is available
*/
bool mcrCheckMsg(HANDLE *hCom);

/*
   Request the value of the internal 89S53 
   timer (in 100s of microseconds). Returns false if 
   serial timeout occurs.
*/
bool mcrGetTimer(HANDLE *hCom, unsigned long *timer);

/*
   Resets event masks, restarts the timer
   Fills in a mask of all buttons pressed right now. 
   Returns false if serial timeout occurs.
*/
bool mcrRestart(HANDLE *hCom, unsigned short *mask);

/*
   Returns all events available in the input buffer. evt is
   malloc-ed for as many events as needed (caller must free after 
   use). Returns false if serial timeout occurs. Note that count might
   still be nonzero indicating that some messages were read, then an
   incomplete message arrived.
*/
bool mcrGetMsgs(HANDLE *hCom, mcrEvent **evt, int *count);

/*
   Stops the microcontroller - restart using mcrRestart(). Returns false
   if serial timeout occurs.
*/
bool mcrStop(HANDLE *hCom);

/* 
  Simulates a pressed button. Takes the current time from the system clock
  if the port is NULL.
*/

void mcrAddMsg(HANDLE *hCom, int button);


#endif